/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from "react";
// Maerial ui components
import Card from "@material-ui/core/Card";
import Paper from "@material-ui/core/Paper";
import CardContent from "@material-ui/core/CardContent";
import Typography from "@material-ui/core/Typography";
import Button from "@material-ui/core/Button";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import EditRoundedIcon from '@material-ui/icons/EditRounded';
import ButtonGroup from "@material-ui/core/ButtonGroup";
import { useStyles } from "../../Styles/DataCard";
//imports from local files
import { GetAData } from "../../../Filters";
import { Link } from "react-router-dom";
import { GetSpecificData } from "../../../Services";
import { apiUrl, apiDomain } from "../../../../env";

const CommunityQuestions = ({ match }) => {
    console.log(match);
    const classes = useStyles();

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [item, setItem] = useState({}); //Necessary to consult the API, data

    useEffect(() => {
        GetSpecificData(GetAData("learning_objects", match.params.id)).then(
            (res) => {
                if (res.state) {
                    setItem(res.data);
                    setIsLoaded(true);
                    setError(false);
                } else {
                    setIsLoaded(true);
                    setError(true);
                }
            }
        );
    }, []);

    if (error) {
        return <div>Houve um erro</div>;
    } else if (!isLoaded) {
        return <div>Loading...</div>;
    } else {
        console.log(item);
        const DATA = [
            {
                subTitle: "ID",
                prop: item.id,
            },
            {
                subTitle: "Nome",
                prop: item.name,
            },
            {
                subTitle: "Descrição",
                prop: item.description,
            },
            {
                subTitle: "Dono(a)",
                prop: item.publisher.name,
            },
            {
                subTitle: "Autor(a)",
                prop: item.author,
            },
            {
                subTitle: "Score",
                prop: item.score,
            },
            {
                subTitle: "Status",
                prop: item.state,
            },
            {
                subTitle: "Criado em",
                prop: item.created_at,
            },
            {
                subTitle: "Atualizado em",
                prop: item.updated_at,
            },
            {
                subTitle: "Criado em",
                prop: item.created_at,
            },
            {
                subTitle: "Visualizações",
                prop: item.views_count,
            },
            {
                subTitle: "Likes",
                prop: item.likes_count,
            },
            {
                subTitle: "Avaliação média",
                prop: item.review_average,
            },
            {
                subTitle: "Likes",
                prop: item.likes_count,
            },
            {
                subTitle: "Linguagem",
                prop: (
                    <div
                        style={{
                            display: "flex",
                            flexDirection: "row",
                            flexWrap: "wrap",
                            justifyContent: "Space-between",
                        }}
                    >
                        {item.language.map((lan, index) => (
                            <Paper
                                elevation={3}
                                key={index}
                                style={{
                                    borderRadius: 18,
                                    padding: "0.8em",
                                    marginBottom: "1em",
                                    marginRight: "1em",
                                    backgroundColor: "#D3D3D3",
                                    fontSize: 14,
                                }}
                            >
                                {lan.name}
                            </Paper>
                        ))}
                    </div>
                ),
            },
            {
                subTitle: "Tipo de recurso",
                prop: item.object_type,
            },
        ];

        return (
            <Card className={classes.root} variant="outlined">
                <CardContent>
                    <div className={classes.displayRow}>
                        <Typography className={classes.title} color="inherit" gutterBottom>
                            {item.name}
                        </Typography>
                        <ButtonGroup
                            color="primary"
                            aria-label="outlined primary button group"
                        >
                            <Link
                                style={{ textDecoration: "none" }}
                                to={`/admin/learningObjects`}
                            >
                                <Button
                                    startIcon={<ListRoundedIcon />}
                                    color="primary"
                                    variant="outlined"
                                >
                                    Listar
                                </Button>
                            </Link>
                            <Button
                                startIcon={<VisibilityIcon />}
                                color="primary"
                                variant="outlined"
                            >
                                <a
                                    style={{ textDecoration: "none" }}
                                    target="_blank"
                                    href={
                                        apiUrl +
                                        "/learning_objects/" +
                                        match.params.id +
                                        "/download"
                                    }
                                >
                                    Ver recurso
                                </a>
                            </Button>
                            <Link
                                style={{ textDecoration: "none" }}
                                to={`/admin/learningObjectEdit/${item.id}`}
                            >
                                <Button
                                    startIcon={<EditRoundedIcon/>}
                                    color="primary"
                                    variant="outlined"
                                >
                                    Editar
                                </Button>
                            </Link>
                        </ButtonGroup>
                    </div>

                    {item.thumbnail ? (
                        <div style={{ marginTop: "1em", marginBottom: "1em" }}>
                            <a target="_blank" href={apiDomain + item.thumbnail}>
                                {" "}
                                <img src={apiDomain + item.thumbnail} alt="" />{" "}
                            </a>
                        </div>
                    ) : null}

                    {DATA.map((info, index) => (
                        <div className={classes.displayColumn} key={index}>
                            <Typography color="initial" className={classes.subTitle}>
                                {info.subTitle}
                            </Typography>
                            <Typography color="textSecondary">
                                {info.prop === null ? "Sem dados" : info.prop}
                            </Typography>
                        </div>
                    ))}
                </CardContent>
            </Card>
        );
    }
};

export default CommunityQuestions;
