/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useRef } from 'react';
//imports material ui components
import { Typography, TextField, Button, Grid } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import CardAction from '@material-ui/core/CardActions';
import ListRoundedIcon from '@material-ui/icons/ListRounded';
import SaveIcon from '@material-ui/icons/Save';
import MenuItem from "@material-ui/core/MenuItem";
//imports local files
import SnackBar from '../../../../Components/SnackbarComponent';
//imports services 
import { Edit, GetSpecificData } from '../../../Services';
import { EditFilter, GetAData } from '../../../Filters';
//routers
import { Link } from 'react-router-dom';
//joadit 
import JoditEditor from 'jodit-react';

const EditCollection = ({ match }) => {
    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete

    const [isLoading, setIsLoading] = useState(false);
    const id = match.params.id
    const [name, setName] = useState('')
    const [privacy, setPrivacy] = useState('')
    const [description, setDescription] = useState('');

    const [errorInName, setErrorInName] = useState({
        error: false,
        message: ''
    })

    const [errorInPrivacy, setErrorInPrivacy] = useState({
        error: false,
        message: ''
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const editor = useRef(null);

    //Configs for jodit
    const config = {
        zIndex: 0,
        readonly: false,
        activeButtonsInReadOnly: ["source", "fullsize", "print", "about"],
        toolbarButtonSize: "middle",
        theme: "default",
        saveModeInCookie: false,
        spellcheck: true,
        editorCssClass: false,
        triggerChangeEvent: true,
        height: 400,
        direction: "ltr",
        debugLanguage: false,
        tabIndex: -1,
        toolbar: true,
        enter: "P",
        useSplitMode: false,
        colorPickerDefaultTab: "background",
        imageDefaultWidth: 100,
        events: {},
        placeholder: "Digite a descrição...",
        showXPathInStatusbar: false,
    };

    const NameHandler = (e) => {
        if (errorInName.error) {
            setErrorInName({
                error: false,
                message: ''
            })
        }
        setName(e.target.value)
    }

    //Verify if the string is empty
    const isEmpty = (text) => {
        return text.length === 0 ? true : false;
    }

    // Fields
    const fields = [
        {
            label: 'ID',
            value: id,
            default: true,
            type: 'text'
        },
        {
            label: 'Nome',
            value: name,
            error: errorInName.error,
            errorMessage: errorInName.message,
            onChange: (event) => NameHandler(event),
            default: false,
            type: 'text',
        },
    ]

    const privacyOptions = [
        {
            value: "private",
            label: "Privado",
        },
        {
            value: "public",
            label: "Público",
        },
    ];

    //handle change of privacy
    const handleChange = (e) => {
        const value = e.target.value;
        setPrivacy(value);
        console.log(privacy)
    };

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const onSubmit = async () => {
        setIsLoading(true)
        if (!isEmpty(name) && !isEmpty(privacy)) {
            const api = EditFilter('collections', id)
            const body = {
                "collection": {
                    'name': name,
                    'privacy': privacy,
                    'description': description
                }
            }
            Edit(api, body).then(res => {
                if (res) {
                    HandleSnack('A Coleção foi alterada com sucesso', true, 'success', '#228B22')
                } else {
                    HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                }
                setIsLoading(false)
            })
        }
        else {
            HandleSnack('Você precisa preencher algumas informações obrigatórias', true, 'warning', '#FFC125')
            if (isEmpty(name)) {
                setErrorInName({
                    error: true,
                    message: 'Este campo precisa ser preenchido'
                })
            }
            if (isEmpty(privacy)) {
                setErrorInPrivacy({
                    error: true,
                    message: 'Este campo precisa ser preenchido'
                })
            }
            setIsLoading(false)
        }
    }

    useEffect(() => {
        const headers = {
            Accept: "application/json",
            "Content-Type": "application/json; charset=utf-8",
            "access-token": sessionStorage.getItem("@portalmec/accessToken"),
            client: sessionStorage.getItem("@portalmec/clientToken"),
            uid: sessionStorage.getItem("@portalmec/uid"),
        };

        GetSpecificData(GetAData("collections", match.params.id), headers).then(
            (res) => {
                if (res.state) {
                    setIsLoaded(true);
                    setError(false);
                    setName(res.data.name)
                    setPrivacy(res.data.privacy)
                    setDescription(res.data.description)
                } else {
                    setIsLoaded(true);
                    setError(true);
                }
            }
        );
    }, []);

    if (error) {
        return <div> Houve um erro... </div>
    } else if (!isLoaded) {
        return <div> Carregando... </div>
    } else {
        return (
            <Card variant='outlined'>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <CardContent>
                    <Grid container direction='row' justify='space-between'>
                        <Typography variant='h4'>
                            {name}
                        </Typography>
                        <Link style={{ textDecoration: 'none' }} to={'/admin/Collections'}>
                            <Button
                                startIcon={<ListRoundedIcon />}
                                variant='outlined'
                                color='primary'
                            >
                                Listar
                            </Button>
                        </Link>
                    </Grid>

                    <div style={{ height: '1em' }}></div>

                    <form style={{ display: 'flex', flexDirection: 'column' }}>
                        {fields.map((field, index) => (
                            <TextField
                                key={index}
                                required={field.required}
                                disabled={field.default}
                                error={field.error}
                                helperText={field.error ? field.errorMessage : ''}
                                style={{marginBottom : '1em'}}
                                label={field.label}
                                value={field.value}
                                onChange={field.onChange}
                                type="search"
                                multiline={true}
                            />
                        ))}
                        <TextField
                            select
                            label="Privacidade"
                            value={privacy ? privacy : ""}
                            error={errorInPrivacy.error}
                            helperText={errorInPrivacy.error ? errorInPrivacy.errorMessage : ''}
                            style={{ marginBottom: '1em' }}
                            onChange={handleChange}
                        >
                            {privacyOptions.map((option, index) => (
                                <MenuItem
                                    key={option.value}
                                    value={option.value}
                                    style={option.value === privacy ? {color : 'blue'} : {color : 'black'}}
                                >
                                    {
                                        option.value
                                    }
                                </MenuItem>
                            ))}
                        </TextField>
                        <JoditEditor
                            ref={editor}
                            value={description}
                            config={config}
                            tabIndex={1} // tabIndex of textarea
                            onBlur={(newContent) => setDescription(newContent.target.innerHTML)} // preferred to use only this option to update the content for performance reasons
                            onChange={(newContent) => { }}
                        />
                    </form>
                </CardContent>
                
                <CardAction>
                    <Button
                        onClick={() => {
                            onSubmit();
                        }}
                        variant="contained"
                        color="primary"
                        disabled={isLoading}
                        startIcon={isLoading ? null : <SaveIcon />}
                    >
                        {
                            isLoading ? <CircularProgress size={24} /> : 'Salvar'
                        }
                    </Button>
                </CardAction>
            </Card>
        )
    }
}

export default EditCollection;