/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useRef } from "react";
import PropTypes from "prop-types";
import SwipeableViews from "react-swipeable-views";
//imports material ui components
import { Typography, TextField, Button, Grid } from "@material-ui/core";
import CircularProgress from "@material-ui/core/CircularProgress";
import Card from "@material-ui/core/Card";
import CardContent from "@material-ui/core/CardContent";
import CardAction from "@material-ui/core/CardActions";
import ListRoundedIcon from "@material-ui/icons/ListRounded";
import List from "@material-ui/core/List";
import ListItem from "@material-ui/core/ListItem";
import { useTheme, makeStyles } from "@material-ui/core/styles";
import Tabs from "@material-ui/core/Tabs";
import Tab from "@material-ui/core/Tab";
import Box from "@material-ui/core/Box";
import SaveIcon from "@material-ui/icons/Save";
import LanguageRoundedIcon from "@material-ui/icons/LanguageRounded";
//imports local files
import SnackBar from "../../../../Components/SnackbarComponent";
//imports services
import { Edit, GetFullList } from "../../../Services";
import { EditFilter, GetAData, Url } from "../../../Filters";
//routers
import { Link } from "react-router-dom";
//jodit
import JoditEditor from "jodit-react";

let currLanPage = 0;
let currLanFilter = "";

let currObjTypePage = 0;
let currObjTypeFilter = "";

const useStyles = makeStyles((theme) => ({
    root: {
        width: "100%",
        maxWidth: 250,
        backgroundColor: theme.palette.background.paper,
        position: "relative",
        overflow: "auto",
        maxHeight: 300,
    },
}));

const EditEducationalObject = ({ match }) => {
    const theme = useTheme();
    const classes = useStyles();

    //Configs for jodit
    const JODIT_CONFIG = {
        zIndex: 0,
        readonly: false,
        activeButtonsInReadOnly: ["source", "fullsize", "print", "about"],
        toolbarButtonSize: "middle",
        theme: "default",
        saveModeInCookie: false,
        spellcheck: true,
        editorCssClass: false,
        triggerChangeEvent: true,
        height: 400,
        direction: "ltr",
        debugLanguage: false,
        tabIndex: -1,
        toolbar: true,
        enter: "P",
        useSplitMode: false,
        colorPickerDefaultTab: "background",
        imageDefaultWidth: 100,
        events: {},
        placeholder: "Digite a descrição...",
        showXPathInStatusbar: false,
    };

    const id = match.params.id;

    const editor = useRef(null);

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [isLoading, setIsLoading] = useState(false); //is loading to submit

    const [value, setValue] = React.useState(0);

    //mutables
    const [name, setName] = useState();
    const [owner, setOwner] = useState();
    const [author, setAuthor] = useState();
    const [listOfLanguages, setListOfLanguages] = useState([]);
    const [languagesID, setLanguagesID] = useState([]);
    const [listOfObjectTypes, setListOfObjectTypes] = useState([]);
    const [objectTypeName, setObjectTypeName] = useState(); 
    const [objectTypeID, setObjectTypeID] = useState(); 
    const [description, setDescription] = useState("");

    // Imutables
    const [CREATE_AT, setCreateAt] = useState("");
    const [UPDATE_AT, setUpdateAt] = useState("");
    const [VIEWS_COUNT, setViewsCount] = useState("");
    const [LIKES_COUNT, setLikesCount] = useState("");
    const [REVIEW_AVERAGE, setReviewAverage] = useState("");
    const [SCORE, setScore] = useState("");

    const [errorInName, setErrorInName] = useState({
        error: false,
        message: "",
    });

    const [errorInOwner, setErrorInOwner] = useState({
        error: false,
        message: "",
    });

    const [errorInAuthor, setErrorInAuthor] = useState({
        error: false,
        message: "",
    });

    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    const NameHandler = (e) => {
        if (errorInName.error) {
            setErrorInName({
                error: false,
                message: "",
            });
        }
        setName(e.target.value);
    };

    const OwnerHandler = (e) => {
        if (errorInOwner.error) {
            setErrorInOwner({
                error: false,
                message: "",
            });
        }
        setOwner(e.target.value);
    };

    const AuthorHandler = (e) => {
        if (errorInAuthor.error) {
            setErrorInAuthor({
                error: false,
                message: "",
            });
        }
        setAuthor(e.target.value);
    }; 

    const onChangeObjectHandler = (e) => {
        currObjTypePage = 0;
        currObjTypeFilter = e.target.value;
        GetFullList(
            Url("object_types", `"name" : "${currObjTypeFilter}"`, currObjTypePage, "DESC")
        ).then((res) => {
            if (res.state) {
                setListOfLanguages([...res.data]);
            } else {
                HandleSnack("Erro ao pegar", true, "warning", "#FA8072");
            }
        });
    }

    const OnChangeLanguageHandler = async (e) => {
        currLanPage = 0;
        currLanFilter = e.target.value;
        GetFullList(
            Url("languages", `"name" : "${currLanFilter}"`, currLanPage, "DESC")
        ).then((res) => {
            if (res.state) {
                setListOfLanguages([...res.data]);
            } else {
                HandleSnack("Erro ao pegar", true, "warning", "#FA8072");
            }
        });
    };

    const HandleLanguageScroll = async (e) => {
        const bottom =
            e.target.scrollHeight - e.target.scrollTop === e.target.clientHeight;
        if (bottom) {
            currLanPage++;
            GetFullList(
                Url("languages", `"name" : "${currLanFilter}"`, currLanPage, "DESC")
            ).then((res) => {
                if (res.state) {
                    if (res.data.length >= 1) {
                        const currData = listOfLanguages.concat(res.data);
                        setListOfLanguages(currData);
                    } else {
                        HandleSnack(
                            "Não há mais linguagens para serem carregadas",
                            true,
                            "warning",
                            "#FFC125"
                        );
                    }
                } else {
                    HandleSnack(
                        "Erro ao carregar as linguagens",
                        true,
                        "warning",
                        "#FA8072"
                    );
                }
            });
        }
    };

    const HandleLanguagePressed = (id) => {
        const currSelectedLanguages = [...languagesID];

        if (currSelectedLanguages[id] === id)
            currSelectedLanguages[id] = undefined
        else
            currSelectedLanguages[id] = id

        setLanguagesID(currSelectedLanguages);
    }

    //Verify if the string is empty
    const isEmpty = (text) => {
        return text.length === 0 ? true : false;
    };

    function TabPanel(props) {
        const { children, value, index, ...other } = props;

        return (
            <div
                role="tabpanel"
                hidden={value !== index}
                id={`full-width-tabpanel-${index}`}
                aria-labelledby={`full-width-tab-${index}`}
                {...other}
            >
                {value === index && <Box style={{ marginTop: "2em" }}>{children}</Box>}
            </div>
        );
    }

    TabPanel.propTypes = {
        children: PropTypes.node,
        index: PropTypes.any.isRequired,
        value: PropTypes.any.isRequired,
    };

    function a11yProps(index) {
        return {
            id: `full-width-tab-${index}`,
            "aria-controls": `full-width-tabpanel-${index}`,
        };
    }

    const handleChange = (event, newValue) => {
        setValue(newValue);
    };

    const handleChangeIndex = (index) => {
        setValue(index);
    };

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    const SaveData = (data) => {
        const languages = [];

        for (let i = 0; i < data.language.length; i++) {
            languages[data.language[i].id] = data.language[i].id;
        } 
        setObjectTypeName(data.object_type);
        setName(data.name);
        setOwner(data.publisher.name);
        setAuthor(data.author);
        setReviewAverage(data.review_average);
        setLikesCount(data.likes_count);
        setViewsCount(data.views_count);
        setScore(data.score);
        setCreateAt(data.created_at);
        setUpdateAt(data.updated_at);
        setDescription(data.description);
        setLanguagesID(languages);
    };

    //handle submit
    const onSubmit = async () => {
        console.log(languagesID);
        setIsLoading(true);
        if (
            !isEmpty(name) &&
            !isEmpty(owner) &&
            !isEmpty(author) &&
            !isEmpty(languagesID) 
        ) {
            const api = EditFilter("learning_objects", id);
            const body = {
                "learning_object": {
                    "author": author,
                    "name": name,
                    "description": description,
                    "object_type_id": 3,
                    "language_ids": languagesID,
                },
            };
            Edit(api, body).then((res) => {
                if (res) {
                    HandleSnack(
                        "O objeto educacional foi alterada com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                } else {
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                }
                setIsLoading(false);
            });
        } else {
            HandleSnack(
                "Você precisa preencher algumas informações obrigatórias",
                true,
                "warning",
                "#FFC125"
            );
            if (isEmpty(name)) {
                setErrorInName({
                    error: true,
                    message: "Este campo precisa ser preenchido",
                });
            }
            setIsLoading(false);
        }
    };

    // Fields
    const fields = [
        {
            label: "ID",
            value: id,
            default: true,
            type: "text",
        },
        {
            label: "Nome",
            value: name,
            error: errorInName.error,
            errorMessage: errorInName.message,
            onChange: (event) => NameHandler(event),
            default: false,
            required: true,
            type: "text",
        },
        {
            label: "Dono(a)",
            value: owner,
            error: errorInOwner.error,
            errorMessage: errorInOwner.message,
            onChange: (event) => OwnerHandler(event),
            default: false,
            required: true,
            type: "text",
        },
        {
            label: "Autor(a)",
            value: author,
            error: errorInAuthor.error,
            errorMessage: errorInAuthor.message,
            onChange: (event) => AuthorHandler(event),
            default: false,
            required: true,
            type: "text",
        },
    ];

    const Imutables = [
        {
            label: "Score",
            value: SCORE,
            default: true,
            type: "text",
        },
        {
            label: "Criado em",
            value: CREATE_AT,
            default: true,
            type: "text",
        },
        {
            label: "Atualizado em",
            value: UPDATE_AT,
            default: true,
            type: "text",
        },
        {
            label: "View count",
            value: VIEWS_COUNT,
            default: true,
            type: "text",
        },
        {
            label: "Likes count",
            value: LIKES_COUNT,
            default: true,
            type: "text",
        },
        {
            label: "Review average",
            value: REVIEW_AVERAGE,
            default: true,
            type: "text",
        },
    ];

    useEffect(() => {
        fetch(GetAData("learning_objects", match.params.id))
            .then((res) => res.json())
            .then(
                (result) => {
                    SaveData(result);
                    setIsLoaded(true);
                },
                (error) => {
                    setIsLoaded(true);
                    setError(true);
                }
            );
    }, []);

    if (error) {
        return <div> Houve um erro... </div>;
    } else if (!isLoaded) {
        return <div> Carregando... </div>;
    } else {
        return (
            <Card variant="outlined">
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() =>
                        setSnackInfo({
                            message: "",
                            icon: "",
                            open: false,
                            color: "",
                        })
                    }
                />
                <CardContent>

                    <Grid container direction="row" justify="space-between">
                        <Typography variant="h4">{name}</Typography>
                        <Link
                            style={{ textDecoration: "none" }}
                            to={"/admin/learningObjects"}
                        >
                            <Button
                                startIcon={<ListRoundedIcon />}
                                variant="outlined"
                                color="primary"
                            >
                                Listar
                            </Button>
                        </Link>
                    </Grid>

                    <Tabs
                        value={value}
                        onChange={handleChange}
                        indicatorColor="primary"
                        textColor="primary"
                        variant="fullWidth"
                        aria-label="full width tabs example"
                    >
                        <Tab label="Resumo" {...a11yProps(0)} />
                        <Tab label="Descrição" {...a11yProps(1)} />
                        <Tab label="Imutáveis" {...a11yProps(2)} />
                    </Tabs>

                    <SwipeableViews
                        axis={theme.direction === "rtl" ? "x-reverse" : "x"}
                        index={value}
                        onChangeIndex={handleChangeIndex}
                    >

                        <TabPanel value={value} index={0} dir={theme.direction}>
                            <form style={{ display: "flex", flexDirection: "column" }}>
                                {fields.map((field, index) => (
                                    <TextField
                                        key={index}
                                        required={field.required}
                                        disabled={field.default}
                                        error={field.error}
                                        helperText={field.error ? field.errorMessage : ""}
                                        style={{ width: "250px", marginBottom: "1em" }}
                                        label={field.label}
                                        defaultValue={field.value}
                                        onBlur={field.onChange}
                                        type="search"
                                        multiline
                                    />
                                ))}

                                <>
                                    <TextField
                                        style={{ width: "250px", marginBottom: "1em" }}
                                        label="Digite o tipo de objeto"
                                        onBlurCapture={onChangeObjectHandler}
                                        type="search"
                                        multiline
                                    />
                                    {listOfObjectTypes.length >= 1 ? (
                                        <List
                                            className={classes.root}
                                            // onScroll={HandleLanguageScroll}
                                        >
                                            {listOfObjectTypes.map((item, index) => (
                                                <ListItem key={item.id} divider>
                                                    <Button
                                                        variant="text"
                                                        startIcon={<LanguageRoundedIcon />}

                                                        color={ objectTypeName === item.name ? 'primary' : 'default'}
                                                    >
                                                        {item.name}
                                                    </Button>
                                                </ListItem>
                                            ))}
                                        </List>
                                    ) : null}
                                </>

                                <>
                                    <TextField
                                        style={{ width: "250px", marginBottom: "1em" }}
                                        label="Digite as línguas"
                                        onBlurCapture={OnChangeLanguageHandler}
                                        type="search"
                                        multiline
                                    />
                                    {listOfLanguages.length >= 1 ? (
                                        <List
                                            className={classes.root}
                                            onScroll={HandleLanguageScroll}
                                        >
                                            {listOfLanguages.map((item, index) => (
                                                <ListItem key={item.id} divider>
                                                    <Button
                                                        variant="text"
                                                        startIcon={<LanguageRoundedIcon />}
                                                        onClick={() => HandleLanguagePressed(item.id)}
                                                        color={languagesID[item.id] === item.id ? 'primary' : 'default'}
                                                    >
                                                        {item.name}
                                                    </Button>
                                                </ListItem>
                                            ))}
                                        </List>
                                    ) : null}
                                </>

                            </form>
                        </TabPanel>

                        <TabPanel value={value} index={1} dir={theme.direction}>
                            <JoditEditor
                                ref={editor}
                                value={description}
                                config={JODIT_CONFIG}
                                tabIndex={1} // tabIndex of textarea
                                onBlur={(newContent) =>
                                    setDescription(newContent.target.innerHTML)
                                } // preferred to use only this option to update the content for performance reasons
                                onChange={(newContent) => { }}
                            />
                        </TabPanel>

                        <TabPanel value={value} index={2} dir={theme.direction}>
                            <form style={{ display: "flex", flexDirection: "column" }}>
                                {Imutables.map((field, index) => (
                                    <TextField
                                        key={index}
                                        disabled={field.default}
                                        style={{ width: "250px", marginBottom: "1em" }}
                                        label={field.label}
                                        defaultValue={field.value}
                                    />
                                ))}
                            </form>
                        </TabPanel>

                    </SwipeableViews>
                </CardContent>

                <CardAction>
                    <Button
                        onClick={() => {
                            onSubmit();
                        }}
                        variant="contained"
                        color="primary"
                        disabled={isLoading}
                        startIcon={isLoading ? null : <SaveIcon />}
                    >
                        {isLoading ? <CircularProgress size={24} /> : "Salvar"}
                    </Button>
                </CardAction>

            </Card>
        );
    }
};

export default EditEducationalObject;
