/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState } from "react";
import moment from "moment";
//imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import { Url } from "../../../Filters";
import { GetFullList } from "../../../Services";
//imports from material ui
import { withStyles } from "@material-ui/core/styles";
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import MenuItem from "@material-ui/core/MenuItem";
import TableRow from "@material-ui/core/TableRow";
import TextField from "@material-ui/core/TextField";
import IconButton from "@material-ui/core/IconButton";
import { Button, Typography, Paper, Grid } from "@material-ui/core";
import CircularProgress from "@material-ui/core/CircularProgress";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import VisibilityIcon from "@material-ui/icons/Visibility";
import LaunchRoundedIcon from "@material-ui/icons/LaunchRounded";
//routers
import { Link } from "react-router-dom";

let currPage = 0;
let currIdFilter;
let currTypeFilter;

const StyledTableCell = withStyles((theme) => ({
    head: {
        backgroundColor: theme.palette.common.black,
        color: theme.palette.common.white,
    },
    body: {
        fontSize: 14,
    },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
    root: {
        "&:nth-of-type(odd)": {
            backgroundColor: theme.palette.action.hover,
        },
    },
}))(TableRow);

const Complaints = () => {
    const PORTAL_MEC = "https://plataformaintegrada.mec.gov.br/";

    const ADD_ONE_LENGHT = [""];
    const TOP_LABELS = [
        "ESTADO DO RECURSO",
        "ID",
        "DESCRIÇÃO",
        "ID OBJETO",
        "DATA(MM/DD/YYYY)",
        "VISUALIZAR",
        "VISITAR",
    ]; //Labels from Table

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
    const [items, setItems] = useState([]); //Necessary to consult the API, data

    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false); //controlls the state of loadind more data
    const [isUpdating, setIsUpdating] = useState(false); //controlls the state of updating data

    const [showFilter, setShowFilter] = useState(false);

    const [option, setOption] = useState("Todos os usuários"); //labels of the text field 'to'

    const [snackInfo, setSnackInfo] = useState({
        message: "",
        icon: "",
        open: false,
        color: "",
    });

    const StateOptions = [
        { id: 0, name: "Pendente" },
        { id: 1, name: "Ativo" },
        { id: 2, name: "Bloqueado / Removido" },
    ];

    const ComplaintReasons = [
        { id: 1, name: "Viola direitos autorais" },
        { id: 2, name: "Conteúdo ofensivo/abusivo" },
        { id: 3, name: "Conta falsa" },
        { id: 4, name: "Spam" },
        { id: 5, name: "Descrição diverge do conteúdo" },
    ];

    //handle snack info
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color,
        });
    };

    //handle load more items
    const LoadMoreItens = async (api) => {
        setIsLoadingMoreItems(true);
        const headers = {
            Accept: "application/json",
            "Content-Type": "application/json; charset=utf-8",
            "access-token": sessionStorage.getItem("@portalmec/accessToken"),
            client: sessionStorage.getItem("@portalmec/clientToken"),
            uid: sessionStorage.getItem("@portalmec/uid"),
            "If-None-Match": null,
        };

        GetFullList(api, headers).then((res) => {
            if (res.state) {
                const arrData = [...res.data];
                if (arrData.length === 0) {
                    HandleSnack(
                        "Não há mais dados para serem carregados",
                        true,
                        "warning",
                        "#FFC125"
                    );
                } else {
                    const arrItems = [...items];
                    arrItems.pop(); //Deleting the last position, that was used to display the button of load more items
                    const arrResult = arrItems.concat(arrData);
                    setItems(arrResult.concat(ADD_ONE_LENGHT));
                }
            } else {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
            }
            setIsLoadingMoreItems(false);
        });
    };

    // handle update list data
    const UpdateHandler = async (api) => {
        setIsUpdating(true);
        const headers = {
            Accept: "application/json",
            "Content-Type": "application/json; charset=utf-8",
            "access-token": sessionStorage.getItem("@portalmec/accessToken"),
            client: sessionStorage.getItem("@portalmec/clientToken"),
            uid: sessionStorage.getItem("@portalmec/uid"),
            "If-None-Match": null,
        };

        GetFullList(api, headers).then((res) => {
            console.log(res);
            if (res.state) {
                HandleSnack(
                    "A lista de dados foi atualizada",
                    true,
                    "success",
                    "#228B22"
                );
                const arrData = [...res.data];
                setItems(arrData.concat(ADD_ONE_LENGHT));
            } else {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
            }
            setIsUpdating(false);
        });
    };

    const handleChange = (e, type) => {
        const value = e.target.value;
        console.log(e);
        setOption(value);
    };

    const ApplyFilter = (id, type) => {
        currPage = 0;
        currIdFilter = id;
        currTypeFilter = type;
        GetFullList(
            Url("complaints", `"${currTypeFilter}" : "${currIdFilter}"`, `${currPage}`, "DESC")
        ).then((res) => {
            if (res.state) {
                const arrData = [...res.data];
                setItems(arrData.concat(ADD_ONE_LENGHT));
                HandleSnack("Filtro aplicado com sucesso", true, "success", "#228B22");
                setIsLoaded(true);
            } else {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsLoaded(true);
            }
        });
    };

    const DescriptionHandler = (e) => {
        currPage = 0;
        currTypeFilter = "description";
        currIdFilter = e.target.value;

        GetFullList(
            Url("complaints", `"${currTypeFilter}" : "${currIdFilter}"`, `${currPage}`, "DESC")
        ).then((res) => {
            if (res.state) {
                const arrData = [...res.data];
                setItems(arrData.concat(ADD_ONE_LENGHT));
                HandleSnack("Filtro aplicado com sucesso", true, "success", "#228B22");
                setIsLoaded(true);
            } else {
                HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                setIsLoaded(true);
            }
        });
    }

    const convertToLink = (type, id) => {
        switch (type) {
            case "LearningObject":
                return `recurso?id=${id}/`;
            case "User":
                return `usuario-publico/${id}/`;
            default:
                return "";
        }
    };

    const ComplaintStatus = (status) => {
        switch (status) {
            case "accepted":
                return (
                    <Paper
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: "#FA8072",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        REMOVIDO
                    </Paper>
                );
            case "complained":
                return (
                    <Paper
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: "#FF8C00",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        PENDENTE
                    </Paper>
                );
            case "rejected":
                return (
                    <Paper
                        style={{
                            textAlign: "center",
                            padding: "0.5em",
                            backgroundColor: "#228B22",
                            fontWeight: "500",
                            color: "#FFFAFA",
                        }}
                    >
                        AVALIADO
                    </Paper>
                );
            default:
                return "NOTHING";
        }
    };

    const DisplayDate = (date) => {
        const convertedData = moment.utc(date);
        return moment(convertedData)
            .format("LLL")
            .toString();
    };

    //getting data from server
    useEffect(() => {
        GetFullList(Url("complaints", "", `${currPage}`, "DESC")).then(
            (res) => {
                if (res.state) {
                    const arrData = [...res.data];
                    setItems(arrData.concat(ADD_ONE_LENGHT));
                    setIsLoaded(true);
                    setError(false);
                } else {
                    HandleSnack("Erro ao carregar os dados", true, "warning", "#FA8072");
                    setIsLoaded(true);
                    setError(true);
                }
            }
        );
    }, []);

    if (error) {
        return <div>Error: {error.message}</div>;
    } else if (!isLoaded) {
        return <div>Loading...</div>;
    } else {
        return (
            <>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() =>
                        setSnackInfo({
                            message: "",
                            icon: "",
                            open: false,
                            color: "",
                        })
                    }
                />

                <Paper style={{ padding: "1em" }}>
                    <Grid container spacing={3} direction="row" alignItems="center">
                        <Grid item xs={6}>
                            <Typography variant="h4">Denúncias</Typography>
                        </Grid>

                        <Grid
                            item
                            xs={6}
                            alignItems="center"
                            justify="center"
                            direction="row"
                        >
                            <Grid container justify="flex-end" spacing={3}>
                                <Grid item>
                                    <Button
                                        variant="contained"
                                        color="secondary"
                                        disabled={isUpdating}
                                        onClick={() => {
                                            currPage = 0;
                                            UpdateHandler(
                                                Url("complaints", "", `${currPage}`, "DESC")
                                            );
                                        }}
                                        startIcon={<UpdateRoundedIcon />}
                                    >
                                        {isUpdating ? <CircularProgress size={24} /> : "Atualizar"}
                                    </Button>
                                </Grid>
                                <Grid item>
                                    <Button
                                        variant="contained"
                                        color="secondary"
                                        onClick={() => {
                                            currPage = 0;
                                            UpdateHandler(
                                                Url("complaints", "", `${currPage}`, "DESC")
                                            );
                                            setShowFilter(!showFilter);
                                        }}
                                        startIcon={<FilterListRoundedIcon />}
                                    >
                                        Filtrar
                                    </Button>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Grid>

                    {showFilter ? (
                        <Grid
                            container
                            direction="row"
                            justify="space-between"
                            alignItems="center" 
                            alignContent="flex-end"
                            spacing={3}
                            xs={12}
                        >
                            <Grid item>
                                <TextField
                                    select
                                    label="Motivo"
                                    value={option ? option : ""}
                                    onChange={handleChange}
                                    helperText="Por favor, selecione uma das opções"
                                >
                                    {ComplaintReasons.map((option, index) => (
                                        <MenuItem
                                            key={option.id}
                                            value={option.name}
                                            name={option.id}
                                            onClick={() =>
                                                ApplyFilter(option.id, "complaint_reason_id")
                                            }
                                        >
                                            {option.name}
                                        </MenuItem>
                                    ))}
                                </TextField>
                            </Grid>
                            <Grid item>
                                <TextField
                                    select
                                    label="Estado"
                                    value={option ? option : ""}
                                    onChange={handleChange}
                                    helperText="Por favor, selecione uma das opções"
                                >
                                    {StateOptions.map((option, index) => (
                                        <MenuItem
                                            key={option.id}
                                            value={option.name}
                                            name={option.id}
                                            onClick={() => ApplyFilter(option.id, "state")}
                                        >
                                            {option.name}
                                        </MenuItem>
                                    ))}
                                </TextField>
                            </Grid>
                            <Grid item>
                                <TextField label="Descrição" onBlur={DescriptionHandler} />
                            </Grid>
                        </Grid>
                    ) : null}
                </Paper>

                <div style={{ height: "2em" }}></div>

                <Grid xs={12} container>
                    <TableData top={TOP_LABELS}>
                        <TableBody>
                            {items.map((row, index) =>
                                index === items.length - 1 ? (
                                    <StyledTableRow style={{ padding: "1em" }}>
                                        {/* Button to load more data */}
                                        <Button
                                            color="primary"
                                            variant="text"
                                            // disabled={isLoadingMoreItems}
                                            startIcon={<AddRoundedIcon />}
                                            disabled={isLoadingMoreItems}
                                            onClick={() => {
                                                currPage++;
                                                if (showFilter) {
                                                    LoadMoreItens(
                                                        Url(
                                                            "complaints",
                                                            `"${currTypeFilter}" : "${currIdFilter}"`,
                                                            `${currPage}`,
                                                            "DESC"
                                                        )
                                                    );
                                                } else {
                                                    LoadMoreItens(
                                                        Url("complaints", "", `${currPage}`, "DESC")
                                                    );
                                                }
                                            }}
                                        >
                                            {isLoadingMoreItems ? (
                                                <CircularProgress size={24} />
                                            ) : (
                                                    "Carregar mais itens"
                                                )}
                                        </Button>
                                    </StyledTableRow>
                                ) : (
                                        <StyledTableRow
                                            key={index}
                                            style={{ flex: 1, width: "100%" }}
                                        >
                                            <StyledTableCell component="th" scope="row">
                                                {ComplaintStatus(row.state)}
                                            </StyledTableCell>
                                            <StyledTableCell align="right">{row.id}</StyledTableCell>
                                            <StyledTableCell align="right">
                                                {row.description}
                                            </StyledTableCell>
                                            <StyledTableCell align="right">
                                                {row.complainable_id}
                                            </StyledTableCell>
                                            <StyledTableCell align="right">
                                                {DisplayDate(row.created_at)}
                                            </StyledTableCell>
                                            <StyledTableCell align="right">
                                                <Link to={`/admin/complaint/${row.id}`}>
                                                    <IconButton>
                                                        <VisibilityIcon style={{ fill: "#00bcd4" }} />
                                                    </IconButton>
                                                </Link>
                                            </StyledTableCell>
                                            <StyledTableCell align="right">
                                                <Button
                                                    variant="text"
                                                    secondary={true}
                                                    startIcon={
                                                        <LaunchRoundedIcon style={{ fill: "#FA8072" }} />
                                                    }
                                                >
                                                    <a
                                                        style={{
                                                            textDecoration: "none",
                                                            color: "#FA8072",
                                                        }}
                                                        target="_blank"
                                                        href={
                                                            PORTAL_MEC +
                                                            convertToLink(
                                                                row.complainable_type,
                                                                row.complainable_id
                                                            )
                                                        }
                                                    >
                                                        MEC RED
                                                    </a>
                                                </Button>
                                            </StyledTableCell>
                                        </StyledTableRow>
                                    )
                            )}
                        </TableBody>
                    </TableData>
                </Grid>
            </>
        );
    }
};
export default Complaints;
